@@ -7,6 +7,7 @@ use futures_util::future::{Either, err, join_all};
 use futures_util::stream::{StreamExt, once, iter};
 use futures_util::FutureExt;
 
+use fluvio_types::defaults::{STORAGE_MAX_BATCH_SIZE, FLUVIO_MAX_SIZE_TOPIC_NAME};
 use fluvio_spu_schema::server::stream_fetch::{
     DefaultStreamFetchRequest, DefaultStreamFetchResponse, GZIP_WASM_API, SMART_MODULE_API,
     LegacySmartModulePayload, SmartModuleWasmCompressed, WASM_MODULE_API, WASM_MODULE_V2_API,
@@ -17,7 +18,7 @@ pub use fluvio_spu_schema::server::stream_fetch::{
 use dataplane::Isolation;
 use dataplane::ReplicaKey;
 use dataplane::ErrorCode;
-use dataplane::batch::Batch;
+use dataplane::batch::{Batch};
 use fluvio_types::event::offsets::OffsetPublisher;
 
 use crate::FluvioError;
@@ -242,7 +243,18 @@ where
             // the records down the consumer stream, THEN an Err with the error inside.
             // This way the consumer always gets to read all records that were properly
             // processed before hitting an error, so that the error does not obscure those records.
-            let batches = response.partition.records.batches.into_iter().map(Ok);
+            let batches = response
+                .partition
+                .records
+                .batches
+                .into_iter()
+                .map(|raw_batch| {
+                    let batch: Result<Batch, _> = raw_batch.try_into();
+                    match batch {
+                        Ok(batch) => Ok(batch),
+                        Err(err) => Err(ErrorCode::Other(err.to_string())),
+                    }
+                });
             let error = {
                 let code = response.partition.error_code;
                 match code {
@@ -339,7 +351,9 @@ where
             .await?;
 
         let ft_stream = async move {
-            if let Some(Ok(response)) = stream.next().await {
+            if let Some(Ok(raw_response)) = stream.next().await {
+                let response: DefaultStreamFetchResponse = raw_response;
+
                 let stream_id = response.stream_id;
 
                 trace!("first stream response: {:#?}", response);
@@ -502,7 +516,7 @@ where
                     .records
                     .batches
                     .iter()
-                    .map(|it| it.records().len())
+                    .map(|it| it.records_len())
                     .sum();
                 let diff = self.remaining - count as i64;
                 self.remaining = diff.max(0);
@@ -562,8 +576,19 @@ mod publish_stream {
 /// MAX FETCH BYTES
 static MAX_FETCH_BYTES: Lazy<i32> = Lazy::new(|| {
     use std::env;
+    use fluvio_protocol::Encoder;
+    use crate::dataplane::fetch::FetchResponse;
+    use crate::dataplane::fetch::FetchableTopicResponse;
+    use crate::dataplane::fetch::FetchablePartitionResponse;
+    use crate::dataplane::batch::MemoryRecords;
     let var_value = env::var("FLV_CLIENT_MAX_FETCH_BYTES").unwrap_or_default();
-    let max_bytes: i32 = var_value.parse().unwrap_or(1000000);
+    let max_bytes: i32 = var_value.parse().unwrap_or_else(|_| {
+        FetchResponse::<MemoryRecords>::default().write_size(0) as i32
+            + FetchableTopicResponse::<MemoryRecords>::default().write_size(0) as i32
+            + FetchablePartitionResponse::<MemoryRecords>::default().write_size(0) as i32
+            + FLUVIO_MAX_SIZE_TOPIC_NAME as i32 // using max size of topic name
+            + STORAGE_MAX_BATCH_SIZE.try_into().unwrap_or(1048588)
+    });
     max_bytes
 });
 
